using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Runtime.InteropServices;
using System.IO;
using System.Xml;
using System.Xml.XPath;
using System.Xml.Xsl;
using System.Net;

namespace BYOJeopardy_net
{
    public partial class FormEdit : Form
    {
        private byoj_treenode undo_nodes = new byoj_treenode();
        private byoj_treenode redo_nodes = new byoj_treenode();

        public FormEdit()
        {
            InitializeComponent();
        }

        private void FormEdit_Load(object sender, EventArgs e)
        {
            has_Undo();
        }

        public void FillTreeView()
        {
            this.treeView1.Nodes.Clear();
            ArrayList Topics = FormMain.jeopardydata.topics;
            for (int t = 0; t < Topics.Count; t++)
            {
                JTopic topic = (JTopic)Topics[t];
                byoj_treenode topicNode = new byoj_treenode(topic.TopicName);
                topicNode.node_type = byoj_node_type.Topic;
                for (int v = 0; v < topic.Values.Count; v++)
                {
                    byoj_treenode valueNode = new byoj_treenode(topic.Values[v].ToString());
                    valueNode.node_type = byoj_node_type.Value;
                    byoj_treenode questionNode = new byoj_treenode();
                    questionNode.node_type = byoj_node_type.Question;
                    byoj_treenode answerNode = new byoj_treenode();
                    answerNode.node_type = byoj_node_type.Answer;
                    if (topic.Questions.Count > v)
                    {
                        questionNode.Text = topic.Questions[v].ToString();
                    }
                    if (topic.Answers.Count > v)
                    {
                        answerNode.Text = topic.Answers[v].ToString();
                    }
                    valueNode.Nodes.Add(questionNode);
                    valueNode.Nodes.Add(answerNode);
                    topicNode.Nodes.Add(valueNode);
                }
                this.treeView1.Nodes.Add(topicNode);
            }
            has_Daily();
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            if (FormMain.FileName == "")
            {
                btnSaveAs_Click(sender, e);
            }
            else
            {
                Save();
            }
        }
        private void btnSaveAs_Click(object sender, EventArgs e)
        {
            if (this.saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                FormMain.FileName = this.saveFileDialog1.FileName;
                Save();
            }
        }
        private void Save()
        {
            FormMain.jeopardydata.topics.Clear();
            for (int t = 0; t < this.treeView1.Nodes.Count; t++)
            {
                byoj_treenode topicNode = (byoj_treenode) this.treeView1.Nodes[t];
                JTopic topic = new JTopic();
                topic.TopicName = topicNode.Text;
                for (int v = 0; v < topicNode.Nodes.Count; v++)
                {
                    byoj_treenode valueNode = (byoj_treenode) topicNode.Nodes[v];
                    topic.Values.Add(valueNode.Text);
                    topic.Questions.Add(valueNode.Nodes[0].Text);
                    topic.Answers.Add(valueNode.Nodes[1].Text);
                }
                FormMain.jeopardydata.topics.Add(topic);
            }
            FormMain.jeopardydata.SaveJeopardy(FormMain.FileName);
        }

        private void btnEditLabel_Click(object sender, EventArgs e)
        {
            this.treeView1.Focus();
            byoj_treenode parentNode = (byoj_treenode)this.treeView1.SelectedNode;
            if (parentNode.node_type == byoj_node_type.Question | parentNode.node_type == byoj_node_type.Answer)
            {
                rtfhtmlEditor.FormEdit frmeditques = new rtfhtmlEditor.FormEdit();
                frmeditques.htmlRichTextBox1.AddHTML(this.treeView1.SelectedNode.Text);
                if (frmeditques.ShowDialog() == DialogResult.OK)
                {
                    this.treeView1.SelectedNode.Text = frmeditques.htmlRichTextBox1.GetHTML(true, false);
                }
            }
            else
            {
                if (this.treeView1.SelectedNode.Text.ToLower() != "daily double")
                {
                    this.treeView1.SelectedNode.BeginEdit();
                }
            }
        }

        private void btnAddTopic_Click(object sender, EventArgs e)
        {
            byoj_treenode topicNode = new byoj_treenode("New Topic");
            topicNode.node_type = byoj_node_type.Topic;
            addTopic(topicNode);
        }

        private void addTopic(byoj_treenode topicNode)
        {  
            this.treeView1.Nodes.Add(topicNode);
            addUndo((byoj_treenode)topicNode, byoj_node_undo.Paste);
            this.treeView1.SelectedNode = topicNode;
            topicNode.BeginEdit();
        }

        private void btnAddValue_Click(object sender, EventArgs e)
        {
             //Add Value Node
                byoj_treenode valueNode = new byoj_treenode("New Value");
                valueNode.node_type = byoj_node_type.Value;
                byoj_treenode questionNode = new byoj_treenode("New Question");
                questionNode.node_type = byoj_node_type.Question;
                byoj_treenode answerNode = new byoj_treenode("New Answer");
                answerNode.node_type = byoj_node_type.Answer;
                valueNode.Nodes.Add(questionNode);
                valueNode.Nodes.Add(answerNode);
              //topicNode.Expand();
              //valueNode.Expand();
                addValue(valueNode, (byoj_treenode)this.treeView1.SelectedNode);
        }

        private void addValue(byoj_treenode valueNode, byoj_treenode parentNode)//parentNode or Selected Node
        {
            //Get parent Topic
            this.treeView1.Focus();

            byoj_treenode topicNode;
            if (parentNode == null)
            {
                return;
            }
            else if (parentNode.node_type == byoj_node_type.Topic)
            {
                topicNode = (byoj_treenode)parentNode;
            }
            else if (parentNode.node_type == byoj_node_type.Value)
            {
                topicNode = (byoj_treenode)parentNode.Parent;
            }
            else  //question or answer node
            {
                topicNode = (byoj_treenode) this.treeView1.SelectedNode.Parent.Parent;
            }

            if (topicNode.Text.ToLower() == "daily double" && topicNode.Nodes.Count > 0)
            {
                return;
            }
            topicNode.Nodes.Add(valueNode);
            addUndo((byoj_treenode)valueNode, byoj_node_undo.Paste);
            this.treeView1.SelectedNode = valueNode;
            valueNode.BeginEdit();
        }

        private void addQuestion_Answer(byoj_treenode questionNode, byoj_treenode parentNode)
        {
            //adds a question or answer
            //tag should equal either "answer" or "question"

            //Get parent Topic
            this.treeView1.Focus();
            
            byoj_treenode valueNode;
            if (parentNode == null)
            {
                return;
            }
            if (parentNode.node_type == byoj_node_type.Value)
            {
                valueNode = (byoj_treenode) this.treeView1.SelectedNode;
            }
            else if (parentNode.node_type == byoj_node_type.Topic)
            {
                valueNode = (byoj_treenode) this.treeView1.SelectedNode.FirstNode;
            }
            else  //question or answer node
            {
                valueNode = (byoj_treenode) this.treeView1.SelectedNode.Parent;
            }

            for (int t = 0; t < valueNode.Nodes.Count; t++)
            {
                byoj_treenode currentNode = (byoj_treenode)valueNode.Nodes[t];
                if (currentNode.node_type == questionNode.node_type)
                {
                    currentNode.Remove();
                }
            }

            valueNode.Nodes.Insert(0, questionNode);
            addUndo((byoj_treenode)questionNode, byoj_node_undo.Paste);
            this.treeView1.SelectedNode = questionNode;
            questionNode.BeginEdit();
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            this.treeView1.SelectedNode.EndEdit(false);

            byoj_treenode selectedNode = (byoj_treenode)this.treeView1.SelectedNode;
            if (selectedNode.node_type == byoj_node_type.Topic | selectedNode.node_type == byoj_node_type.Value)
            {
                if (selectedNode.node_type == byoj_node_type.Value && selectedNode.Parent.Text.ToLower() == "daily double")
                {
                    //If Daily double value
                    addUndo((byoj_treenode)this.treeView1.SelectedNode.Parent, byoj_node_undo.Cut);
                    this.treeView1.SelectedNode.Parent.Remove();
                }
                else
                {
                    addUndo((byoj_treenode)this.treeView1.SelectedNode, byoj_node_undo.Cut);
                    this.treeView1.SelectedNode.Remove();
                }
            }
            has_Daily();
        }

        private void btnAddDaily_Click(object sender, EventArgs e)
        {
            if (!has_Daily())
            {
                byoj_treenode topicNode = new byoj_treenode("Daily Double");
                topicNode.node_type = byoj_node_type.Topic;
                this.treeView1.Nodes.Add(topicNode);

                //Add Value Node
                byoj_treenode valueNode = new byoj_treenode("New Topic");
                valueNode.node_type = byoj_node_type.Value;
                byoj_treenode questionNode = new byoj_treenode("New Question");
                questionNode.node_type = byoj_node_type.Question;
                byoj_treenode answerNode = new byoj_treenode("New Answer");
                answerNode.node_type = byoj_node_type.Answer;
                valueNode.Nodes.Add(questionNode);
                valueNode.Nodes.Add(answerNode);
                topicNode.Nodes.Add(valueNode);
                topicNode.Expand();
                valueNode.Expand();
                addUndo((byoj_treenode)valueNode, byoj_node_undo.Paste);
                this.treeView1.SelectedNode = valueNode;
                valueNode.BeginEdit();
                this.btnAddDaily.Enabled = false;
            }
        }

        private bool has_Daily()
        {
            for (int t = 0; t < this.treeView1.Nodes.Count; t++)
            {
                if (this.treeView1.Nodes[t].Text.ToLower() == "daily double" && this.treeView1.Nodes[t].Nodes.Count > 0)
                {
                    this.btnAddDaily.Enabled = false;
                    this.addDailyDoubleToolStripMenuItem.Enabled = false;
                    return true;
                }
            }
            this.btnAddDaily.Enabled = true;
            this.addDailyDoubleToolStripMenuItem.Enabled = true;
            return false;
        }

        FormExport form_export;
        private FileDialog savedialog_export;
        private void btnExport_Click(object sender, EventArgs e)
        {
            if (form_export == null)
            {
                form_export = new FormExport();

                string dir = Application.StartupPath + "\\" + "export";
                string[] xsllist = System.IO.Directory.GetFiles(dir, "*.xsl");
                for (int t = 0; t < xsllist.Length; t++)
                {
                    XmlDataDocument doc = new XmlDataDocument();
                    doc.Load(xsllist[t]);
                    XmlNode docType = doc.FirstChild;
                    XmlNode rootElement = docType.NextSibling;
                    byoj_listviewitem template = new byoj_listviewitem();
                    template.filename = xsllist[t];
                    for (int c = 0; c < rootElement.ChildNodes.Count; c++)
                    {
                        XmlNode node = rootElement.ChildNodes[c];
                        if (node.Name == "xs:template-name")
                        {
                            template.name = node.InnerText;
                            template.Text = node.InnerText;
                        }
                        if (node.Name == "xs:template-description")
                        {
                            template.description = node.InnerText;
                        }
                        if (node.Name == "xs:template-version")
                        {
                            template.version = node.InnerText;
                        }
                        if (node.Name == "xs:template-image")
                        {
                            template.image = node.InnerText;
                        }
                    }
                    form_export.listView1.Items.Add(template);
                }
            }
            if(form_export.ShowDialog() == DialogResult.OK)
            {
                if (savedialog_export == null)
                {
                    savedialog_export = new SaveFileDialog();
                    savedialog_export.Filter = "HTML Files(*.html;*.htm)|*.html;*.htm|All Files|*.*";
                }
                if (savedialog_export.ShowDialog() == DialogResult.OK)
                {
                    //open xml file (byojeopardy file)
                    string temp_file = Path.GetTempFileName();
                    FormMain.jeopardydata.SaveJeopardy(temp_file);
                    XPathDocument myXPathDoc = new XPathDocument(temp_file);

                    //open xsl file
                    byoj_listviewitem template = (byoj_listviewitem)form_export.listView1.SelectedItems[0];
                    XslCompiledTransform xslt = new XslCompiledTransform();
                    xslt.Load(template.filename);

                    //add arguments to pass to xsl
                    XsltArgumentList argList = new XsltArgumentList();
                    argList.AddParam("xsltFileName", "", template.filename);
                    argList.AddParam("OutputFileName", "", Path.GetFileNameWithoutExtension(savedialog_export.FileName));
                    argList.AddParam("answer_in_form_of_question", "", Convert.ToInt16(form_export.AnswerasQuestion.Checked));

                    //create stream for output
                    XmlTextWriter writer = new XmlTextWriter(savedialog_export.FileName, null);

                    // Execute the transform.
                    xslt.Transform(myXPathDoc, argList, writer);
                    writer.Close();
                }
            }
        }

        private byoj_treenode node_copy;
        private void cutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            byoj_treenode selectedNode = (byoj_treenode)this.treeView1.SelectedNode;
            if (selectedNode.node_type == byoj_node_type.Topic | selectedNode.node_type == byoj_node_type.Value)
            {
                addUndo(selectedNode, byoj_node_undo.Cut);
                node_copy = (byoj_treenode)selectedNode.CloneAll();
                selectedNode.Remove();
            }
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            byoj_treenode selectedNode = (byoj_treenode)this.treeView1.SelectedNode;
            node_copy = selectedNode.CloneAll();
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            byoj_treenode paste_node = node_copy.CloneAll();
            if (paste_node.node_type == byoj_node_type.Topic)
            {
                addTopic(paste_node);
            }
            else if (paste_node.node_type == byoj_node_type.Value)
            {
                addValue(paste_node, (byoj_treenode)this.treeView1.SelectedNode);
            }
            else
            {
                addQuestion_Answer(paste_node, (byoj_treenode) this.treeView1.SelectedNode);
            }
            addUndo((byoj_treenode)this.treeView1.SelectedNode, byoj_node_undo.Paste);
            this.treeView1.SelectedNode = paste_node;
        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btnDelete_Click(sender, e);
        }

        private void editLabelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btnEditLabel_Click(sender, e);
        }

        private void addTopicToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btnAddTopic_Click(sender, e);
        }

        private void addValueToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btnAddValue_Click(sender, e);
        }

        private void addDailyDoubleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btnAddDaily_Click(sender, e);
        }




        private void has_Undo()
        {
            if (undo_nodes.Nodes.Count > 0)
            {
                this.undoToolStripMenuItem.Enabled = true; 
            }
            else
            {
                this.undoToolStripMenuItem.Enabled = false; 
            }

            if (redo_nodes.Nodes.Count > 0)
            {
                this.redoToolStripMenuItem.Enabled = true; 
            }
            else
            {
                this.redoToolStripMenuItem.Enabled = false; 
            }
        }


        private void addUndo(byoj_treenode node, byoj_node_undo undo_type)
        {
            byoj_treenode undo = (byoj_treenode)node.CloneAll();
            undo.node_undo = undo_type;
            undo.origin_parent_node = (byoj_treenode)node.Parent;
            undo.origin_index = node.Index;
            undo.origin_selected_node = (byoj_treenode)this.treeView1.SelectedNode;
            undo_nodes.Nodes.Add(undo);
            redo_nodes.Nodes.Clear();
            has_Undo();
        }

        private void undoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.treeView1.Focus();
            byoj_treenode node = (byoj_treenode)undo_nodes.Nodes[undo_nodes.Nodes.Count - 1];
            byoj_treenode node_clone = node.CloneAll();
            if (node.node_undo == byoj_node_undo.Cut)
            {
                if (node.origin_parent_node != null)
                {
                    node.origin_parent_node.Nodes.Insert(node.origin_index, node_clone);
                }
                else
                {
                    this.treeView1.Nodes.Insert(node.origin_index, node_clone);
                }
                this.treeView1.SelectedNode = node_clone;
            }
            if (node.node_undo == byoj_node_undo.Paste)
            {
                if (node.origin_parent_node != null)
                {
                    node.origin_parent_node.Nodes.RemoveAt(node.origin_index);
                }
                else
                {
                    this.treeView1.Nodes.RemoveAt(node.origin_index);
                }
                if (node.origin_selected_node != null)
                {
                    this.treeView1.SelectedNode = node.origin_selected_node;
                }
            }
            undo_nodes.Nodes[undo_nodes.Nodes.Count - 1].Remove();
            redo_nodes.Nodes.Add(node);
            has_Undo();
        }

        private void redoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.treeView1.Focus();
            byoj_treenode node = (byoj_treenode)redo_nodes.Nodes[redo_nodes.Nodes.Count - 1];
            byoj_treenode node_clone = node.CloneAll();
            if (node.node_undo == byoj_node_undo.Paste)
            {
                if (node.origin_parent_node != null)
                {
                    node.origin_parent_node.Nodes.Insert(node.origin_index, node_clone);
                }
                else
                {
                    this.treeView1.Nodes.Insert(node.origin_index, node_clone);
                }
                this.treeView1.SelectedNode = node_clone;
            }
            if (node.node_undo == byoj_node_undo.Cut)
            {
                if (node.origin_parent_node != null)
                {
                    node.origin_parent_node.Nodes.RemoveAt(node.origin_index);
                }
                else
                {
                    this.treeView1.Nodes.RemoveAt(node.origin_index);
                }
                if (node.origin_selected_node != null)
                {
                    this.treeView1.SelectedNode = node.origin_selected_node;
                }
            }
            redo_nodes.Nodes[redo_nodes.Nodes.Count - 1].Remove();
            undo_nodes.Nodes.Add(node);
            has_Undo();
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
            this.Dispose();
        }

        private void exportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btnExport_Click(sender, e);
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormTemplate frmTemplate = new FormTemplate();
            if (frmTemplate.ShowDialog() == DialogResult.OK)
            {
                FormMain.FileName = "";
                if (frmTemplate.rbtnSingle.Checked | frmTemplate.rbtnDouble.Checked)
                {
                    FormMain.jeopardydata.CreateJeopardy(frmTemplate.rbtnDouble.Checked);
                    this.FillTreeView();
                }
            }
        }
    }

    public enum byoj_node_undo { Delete, Insert, Cut, Paste };
    public enum byoj_node_type { Topic, Value, Question, Answer, Daily_Double };

   public partial class byoj_treenode : TreeNode
    {
        public byoj_node_type node_type;
        public byoj_node_undo node_undo;
        public byoj_treenode origin_parent_node;
        public byoj_treenode origin_selected_node;
        public int origin_index;

       public byoj_treenode()
       {
       }

       public byoj_treenode(string name)
       {
           this.Text = name;
       }

       public byoj_treenode CloneAll()
       {
           return CloneAll(this);
       }

       private byoj_treenode CloneAll(byoj_treenode parent)
       {
           byoj_treenode myClone = (byoj_treenode)parent.Clone();
           myClone.node_type = parent.node_type;
           myClone.node_undo = parent.node_undo;
           myClone.origin_parent_node = parent.origin_parent_node;
           myClone.origin_selected_node = parent.origin_selected_node;
           myClone.origin_index = parent.origin_index;
           for (int n = 0; n < parent.Nodes.Count; n++)
           {
               myClone.Nodes[n] = CloneAll((byoj_treenode) parent.Nodes[n]);
           }
           return myClone;
       }

    }
}